with Ada.Finalization;
use  Ada.Finalization;
with Video.Pixels.Indexed, Video.Colors.Palettes, Video.Colors.Maps;
use  Video.Pixels.Indexed, Video.Colors.Palettes, Video.Colors.Maps;

package Video.Images.Raster.Indexed is

	type Indexed_Raster_Image is limited interface and Raster_Image and Constant_Color_Map;

	function Pixel_Index (
		Source : Indexed_Raster_Image;
		A      : Point)
		return   Color_Index is abstract;
	-- A pixel index value, an untyped way to read it

	-- note: Function is named `Pixel_Index` to avoid mixed it with `Pixel` group
	--       As it returns integer type it can break type safety

	type Image_with_Shared_Palette is abstract new Controlled and Indexed_Raster_Image with private;
	-- Base for Image using externally provided palette

	-- Descend from controlled: there likely to be controlled types in the class

	overriding function Map_Color (
		Source : Image_with_Shared_Palette;
		Index  : Color_Index)
		return   Color;

	function To_Palette (
		Source : Image_with_Shared_Palette)
		return   Palette;

	--

	package Index_1 is new Generic_Raster_Images (
		Pixel_Type => Index_1_Pixel,
		Array_Type => Index_1_Array,
		Parent     => Indexed_Raster_Image);

	package Index_4 is new Generic_Raster_Images (
		Pixel_Type => Index_4_Pixel,
		Array_Type => Index_4_Array,
		Parent     => Indexed_Raster_Image);

	package Index_8 is new Generic_Raster_Images (
		Pixel_Type => Index_8_Pixel,
		Array_Type => Index_8_Array,
		Parent     => Indexed_Raster_Image);

private

	type Image_with_Shared_Palette is abstract new Controlled and Indexed_Raster_Image with record
		Color_Map : access constant Palette;
	end record;

end Video.Images.Raster.Indexed;
