with Video.Integer_Geometry;
use  Video.Integer_Geometry;

--with Ada.Text_IO;
--use  Ada.Text_IO;
--with Ada.Integer_Text_IO;
--use  Ada.Integer_Text_IO;

package body Video.Pixels.Generic_Blits is

	procedure Fill_Rectangle (
		Color  : in     Pixel;
		Target : in out Pixel_Array;
		Bounds : in     Video.Integer_Geometry.Box)
	is
	begin
		for Y in Bounds.Y.First .. Bounds.Y.Last - 1 loop
			for X in Bounds.X.First .. Bounds.X.Last - 1 loop
				Target (Y, X) := Color;
			end loop;
		end loop;
	end;

	procedure Line (
		Color  : in     Pixel;
		Target : in out Pixel_Array;
		A, B   : Integer_Geometry.Point)
	is
		D    : constant Integer_Geometry.Point := B - A;
		S    : constant Integer_Geometry.Point := (
			X => (if D.X >= 0 then 1 else -1),
			Y => (if D.Y >= 0 then 1 else -1));
		N, M : Integer;
		E    : Integer;
		P    : Integer_Geometry.Point := A;
	begin
		if abs D.X > abs D.Y then -- Nearly horizontal
			N := abs D.X;
			M := abs D.Y;
			E := -N / 2;
			for I in 1 .. N loop
				E := E + M;
				if E >= 0 then
					E := E - N;
					P.Y := P.Y + S.Y;
				end if;
				P.X := P.X + S.X;
				Target (P.Y, P.X) := Color;
			end loop;
		else -- Nearly vertical
			N := abs D.Y;
			M := abs D.X;
			E := -N / 2;
			for I in 1 .. N loop
				E := E + M;
				if E >= 0 then
					E := E - N;
					P.X := P.X + S.X;
				end if;
				P.Y := P.Y + S.Y;
				Target (P.Y, P.X) := Color;
			end loop;
		end if;
	end;

	procedure Circle (
		Color  : in     Pixel;
		Target : in out Pixel_Array;
		Center : in     Integer_Geometry.Point;
		Radius : in     Natural)
	is
		C : Integer := Radius; -- 'cosine' coordinate
		S : Integer := 0;      -- 'sine' coordinate
		D : Integer := Radius; -- (R**2 - S**2) - (C - 1/2)**2
	begin
		pragma Assert (D = Radius**2 - Radius * (Radius - 1));
		Target (Center.Y, Center.X + Radius) := Color;
		Target (Center.Y, Center.X - Radius) := Color;
		Target (Center.Y + Radius, Center.X) := Color;
		Target (Center.Y - Radius, Center.X) := Color;
		loop
			D := D - 2 * S - 1; -- S**2 - (S + 1)**2
			S := S + 1;
			pragma Assert (D = Radius**2 - S**2 - C * (C - 1));
			if D < 0 then
				C := C - 1;
				D := D + 2 * C; -- (C - 1/2)**2 - (C - 3/2)**2
			end if;
			pragma Assert (D = Radius**2 - S**2 - C * (C - 1));
			exit when C < S;
			Target (Center.Y + C, Center.X + S) := Color;
			Target (Center.Y + C, Center.X - S) := Color;
			Target (Center.Y - C, Center.X + S) := Color;
			Target (Center.Y - C, Center.X - S) := Color;
			Target (Center.Y + S, Center.X + C) := Color;
			Target (Center.Y + S, Center.X - C) := Color;
			Target (Center.Y - S, Center.X + C) := Color;
			Target (Center.Y - S, Center.X - C) := Color;
		end loop;
	end Circle;

	procedure Ellipse (
		Color  : in     Pixel;
		Target : in out Pixel_Array;
		Bounds : in     Integer_Geometry.Box)
	is
		subtype Prod is Coordinate_Product;
		X, Y   : Coordinate; -- Offset from center in half-pixels
		D      : Prod; -- Deviation from ideal ellipse
		A, B   : Distance; -- Axis lengths
		A2, B2 : Prod; -- Squares of axis length
		S      : Prod;
		procedure Draw is
			XS : constant Coordinate := Bounds.X.First + Bounds.X.Last;
			YS : constant Coordinate := Bounds.Y.First + Bounds.Y.Last;
		begin
			Target ((YS + Y) / 2, (XS + X) / 2) := Color;
			Target ((YS + Y) / 2, (XS - X) / 2) := Color;
			Target ((YS - Y) / 2, (XS + X) / 2) := Color;
			Target ((YS - Y) / 2, (XS - X) / 2) := Color;
		end;
	begin
		if Is_empty (Bounds) then
			return; -- No pixels to draw
		end if;

		A := Length (Bounds.X); A2 := Prod (A)**2;
		B := Length (Bounds.Y); B2 := Prod (B)**2;

		-- Horizontal part
		if A mod 2 = 0 then
			Target (Center (Bounds.Y), Bounds.X.First) := Color;
			Target (Center (Bounds.Y), Bounds.X.Last)  := Color;
			X := 0;
		else
			X := -1;
		end if;

		Y := B;

		-- D := ((DX * B)**2 + (DY * A)**2 - A**2 * B) / 4;
		D := ((Prod (X))**2 * B2 + 2) / 4; -- + 2 is for rounding

		while Prod (X) * B2 < Prod (Y) * A2 loop
			D := D + (Prod (X) + 1) * B2;
			X := X + 2;
			--pragma Assert (D = (X * B)**2 + (Y * A)**2 - (A * B)**2);
			-- Step Y -> Y - 2
			-- D(i)   = ((X * B)**2 + ( Y      * A)**2 - A**2 * B**2) / 4
			-- D(i+1) = ((X * B)**2 + ((Y - 2) * A)**2 - A**2 * B**2) / 4
			-- D(i+1) - D(i) = (((Y - 2) * A)**2) / 4 - ((Y * A)**2) / 4
			--               = A**2 * (1 - Y)
			S := (1 - Prod (Y)) * A2;
			-- Step down when its deviation is of opposite sign and less in abs. value
			if D + S >= -D then
				Y := Y - 2;
				D := D + S;
				--pragma Assert (D = (X * B)**2 + (Y * A)**2 - (A * B)**2);
			end if;
			Draw;
		end loop;

		-- Vertical part
		while Y > 2 loop
			D := D + (1 - Prod (Y)) * A2;
			Y := Y - 2;
			--pragma Assert (D = (X * B)**2 + (Y * A)**2 - (A * B)**2);
			-- Step X -> X + 2
			-- D(i)   = (( X      * B)**2 + (Y * A)**2 - A**2 * B**2) / 4
			-- D(i+1) = (((X + 2) * B)**2 + (Y * A)**2 - A**2 * B**2) / 4
			-- D(i+1) - D(i) = (((X + 2) * B)**2) / 4 - ((X * B)**2) / 4
			--               = B**2 * (X + 1)
			S := (Prod (X) + 1) * B2;
			if D + S / 2 < -D then
				X := X + 2;
				D := D + S;
				--pragma Assert (D = (X * B)**2 + (Y * A)**2 - (A * B)**2);
			end if;
			Draw;
		end loop;

		if B mod 2 = 0 then
			Target (Bounds.Y.First, Center (Bounds.X)) := Color;
			Target (Bounds.Y.Last,  Center (Bounds.X)) := Color;
		end if;

	end Ellipse;

	-- Ellipse equation:
	-- (X - Xc)²/A² + (Y - Yc)²/B² = 1
	-- in integers:
	-- (X - Xc)² B² + (Y - Yc)² A² = A² B²

end Video.Pixels.Generic_Blits;
