with Video.Renderables;
with Video.Integer_Geometry;

generic
	type Pixel is private;
	type Pixel_Array is array (Integer range <>, Integer range <>) of Pixel;
	with package Pixel_Renderables is new Video.Renderables.Paint_Renderables (Pixel);
package Video.Pixels.Generic_Renderers is

	type Renderer (Target : access Pixel_Array) is limited new Pixel_Renderables.Renderable with null record;

	overriding function Bounding_Box (S : Renderer) return Integer_Geometry.Box
	is (X => (S.Target'First (2), S.Target'Last (2) - 1), Y => (S.Target'First (1), S.Target'Last (1) - 1));

	overriding procedure Clear (
		Target : in out Renderer;
		Paint  : in     Pixel);

	overriding procedure Write_Pixel (
		Target : in out Renderer;
		A      : in     Integer_Geometry.Point;
		Paint  : in     Pixel) with Inline;

	overriding procedure Fill_Rectangle (
		Target : in out Renderer;
		Bounds : in     Integer_Geometry.Box;
		Paint  : in     Pixel);

	overriding procedure Line (
		Target : in out Renderer;
		A, B   : in     Integer_Geometry.Point;
		Paint  : in     Pixel);

	procedure Circle (
		Target : in out Renderer;
		Center : in     Integer_Geometry.Point;
		Radius : in     Natural;
		Paint  : in     Pixel);

	procedure Ellipse (
		Target : in out Renderer;
		Bounds : in     Integer_Geometry.Box;
		Paint  : in     Pixel);

end Video.Pixels.Generic_Renderers;
