with Video.Integer_Geometry;
use  Video.Integer_Geometry;

package Video.Renderables with Pure is
	-- Basic rendering interface.

	type Root_Renderable is limited interface;
	-- Basic type for an entity that can render.
	-- It can be
	-- - A canvas with rendering procedures attached
	-- - A rendering session with target locked
	-- - A hardware device driver (display, printer)

	function Bounding_Box (S : Root_Renderable) return Box is abstract;
	-- Coordinate limits of a surface

	procedure Sync (S : in out Root_Renderable) is null;
	-- Make painting available to outer world by e.g.
	-- - pushing data to the attached graphic device,
	-- - waiting for painting process to be complete.
	--
	-- Alternatively synchronization should be done
	-- on the destruction of the renderable object

	generic
		type Paint_Type is private;
		-- This is a type for something to fill area with
	package Paint_Renderables is

		type Renderable is limited interface and Root_Renderable;

		procedure Clear (
			Target : in out Renderable;
			Paint  :        Paint_Type) is abstract;

		procedure Write_Pixel (
			Target : in out Renderable;
			A      : in     Integer_Geometry.Point;
			Paint  : in     Paint_Type) is abstract;

		procedure Fill_Rectangle (
			Target : in out Renderable;
			Bounds : in     Integer_Geometry.Box;
			Paint  : in     Paint_Type) is abstract;

		procedure Line (
			Target : in out Renderable;
			A, B   : in     Integer_Geometry.Point;
			Paint  : in     Paint_Type) is abstract;
	end Paint_Renderables;

end Video.Renderables;
