with Ada.Text_IO, Ada.Integer_Text_IO;
use  Ada.Text_IO, Ada.Integer_Text_IO;
with Interfaces.C;
with SDL.Video.Windows.Makers;
with SDL.Video.Surfaces.Makers;
with SDL.Events.Events;
with SDL.Images.IO;
with Video.Colors;
with Video.Integer_Geometry;
with Video.SDL2.Surfaces;
with Video.Pixels.RGB.Renderers;
with Example_Scene;

procedure Example_SDL_Raster is

	procedure Create_From_Array is new SDL.Video.Surfaces.Makers.Create_From_Array (
		Element       => Video.Pixels.RGB.RGB565_Pixel,
		Index         => Integer,
		Element_Array => Video.Pixels.RGB.RGB565_Array);

	Screen_Width  : constant := 640;
	Screen_Height : constant := 480;
	Wnd           : aliased SDL.Video.Windows.Window;
	Screen        : aliased Video.Pixels.RGB.RGB565_Array := (0 .. Screen_Height - 1 =>  (0 .. Screen_Width - 1 => <>));
	Img           : Video.SDL2.Surfaces.SDL_Surface;

	Ev            : SDL.Events.Events.Events;
	use type SDL.Events.Event_Types;

begin
	if not SDL.Initialise (Flags => SDL.Enable_Screen) then
		raise Program_Error;
	end if;

	SDL.Video.Windows.Makers.Create (
		Win      => Wnd,
		Title    => "Raster Draw",
		Position => SDL.Natural_Coordinates'(
			SDL.Video.Windows.Centered_Window_Position,
			SDL.Video.Windows.Centered_Window_Position),
		Size     => SDL.Positive_Sizes'(Screen_Width, Screen_Height),
		Flags    => 0);

	Create_From_Array (Img.Data, Screen'Access,
		Red_Mask => 16#F800#, Green_Mask => 16#07E0#, Blue_Mask => 16#001F#, Alpha_Mask => 0);

	declare
		Rend          : Video.Pixels.RGB.Renderers.RGB565_Color_Renderers.Color_Renderer (Screen'Access);
	begin
		Example_Scene (Rend);

		for X in 0 .. 10 loop
			Rend.Circle ((Screen_Width / 2, Screen_Height / 2), X * 10,
				Color => Video.Colors.Hex_6 (16#FF00FF#));
		end loop;

		for X in 0 .. 10 loop
			Rend.Ellipse (((Screen_Width / 2 - X * 20, Screen_Width / 2 + X * 20),
			               (Screen_Height / 2 - 200 + X * 20, Screen_Height / 2 + 200 - X * 20)),
			              Color => Video.Colors.Hex_6 (16#00FFFF#));
		end loop;
	end;

	declare
		Surf          : Video.SDL2.Surfaces.Window_Surface (Wnd'Access);
	begin
		Surf.Paste ((0, 0), Img);

	end;

	Wait : loop
		while SDL.Events.Events.Poll (Ev) loop
			exit Wait when Ev.Common.Event_Type = SDL.Events.Quit;
		end loop;
	end loop Wait;

	Wnd.Finalize;

	SDL.Finalise;
end Example_SDL_Raster;
