with SDL.Video.Rectangles;
with Video.Integer_Geometry;
with Video.SDL2.Rectangles;
with Video.SDL2.Colors;

use type Video.Integer_Geometry.Box;

package body Video.SDL2.Renderers is

	procedure Finalize (Target : in out SDL_Renderer) is
	begin
		Target.Sync;
	end Finalize;

	procedure Sync (Target : in out SDL_Renderer) is
	begin
		Target.Data.Present;
	end Sync;

	function Bounding_Box (R : SDL_Renderer) return Integer_Geometry.Box is
		Rect : SDL.Video.Rectangles.Rectangle;
	begin
		R.Data.Get_Viewport (Rect);
		return Video.SDL2.Rectangles.From_SDL (Rect);
	end Bounding_Box;

	procedure Clear (
		Target : in out SDL_Renderer;
		Color  :        Video.Colors.Color)
	is
	begin
		Target.Data.Set_Draw_Colour (Video.SDL2.Colors.To_SDL (Color));
		Target.Data.Clear;
	end Clear;

	procedure Write_Pixel (
		Target : in out SDL_Renderer;
		A      :        Integer_Geometry.Point;
		Color  :        Video.Colors.Color)
	is
	begin
		Target.Data.Set_Draw_Colour (Video.SDL2.Colors.To_SDL (Color));
		Target.Data.Draw (Rectangles.To_SDL (A));
	end Write_Pixel;

	procedure Fill_Rectangle (
		Target : in out SDL_Renderer;
		Bounds :        Integer_Geometry.Box;
		Color  :        Video.Colors.Color)
	is
	begin
		Target.Data.Set_Draw_Colour (Video.SDL2.Colors.To_SDL (Color));
		Target.Data.Fill (Rectangles.To_SDL (Bounds));
	end Fill_Rectangle;

	procedure Line (
		Target : in out SDL_Renderer;
		A, B   : in     Integer_Geometry.Point;
		Paint  : in     Video.Colors.Color)
	is
	begin
		Target.Data.Set_Draw_Colour (Video.SDL2.Colors.To_SDL (Paint));
		Target.Data.Draw (Rectangles.To_SDL_Line_Segment (A, B));
	end;

	procedure Paste (
		Target : in out SDL_Renderer;
		A      :        Integer_Geometry.Point;
		Source :        Textures.SDL_Texture)
	is
	begin
		Target.Data.Copy (Source.Data, Rectangles.To_SDL (Source.Bounding_Box + A));
	end Paste;

end Video.SDL2.Renderers;
