with Ada.Finalization;
with SDL.Video.Renderers;
with Video.Colors;
with Video.Integer_Geometry;
with Video.Renderables.Color;
with Video.SDL2.Textures;

package Video.SDL2.Renderers is

	type SDL_Renderer is limited new Ada.Finalization.Limited_Controlled
	and Video.Renderables.Color.Renderable
	and Textures.Texture_Renderables.Sprite_Renderable with record
		Data : SDL.Video.Renderers.Renderer;
	end record;

	overriding function Bounding_Box (R : SDL_Renderer) return Integer_Geometry.Box;

	overriding procedure Clear (
		Target : in out SDL_Renderer;
		Color  : in     Video.Colors.Color);

	overriding procedure Write_Pixel (
		Target : in out SDL_Renderer;
		A      : in     Integer_Geometry.Point;
		Color  : in     Video.Colors.Color);

	overriding procedure Fill_Rectangle (
		Target : in out SDL_Renderer;
		Bounds : in     Integer_Geometry.Box;
		Color  : in     Video.Colors.Color);

	overriding procedure Line (
		Target : in out SDL_Renderer;
		A, B   : in     Integer_Geometry.Point;
		Paint  : in     Video.Colors.Color);

	overriding procedure Paste (
		Target : in out SDL_Renderer;
		A      : in     Integer_Geometry.Point;
		Source : in     Textures.SDL_Texture);

	overriding procedure Finalize (Target : in out SDL_Renderer);

	overriding procedure Sync (Target : in out SDL_Renderer);

end Video.SDL2.Renderers;
