with SDL.Video.Pixel_Formats;
with SDL.Video.Palettes;
with SDL.Video.Rectangles;
with Video.SDL2.Colors;
with Video.SDL2.Rectangles;
with Video.Algorithms.Generic_Line;

package body Video.SDL2.Surfaces is
	use type Video.Integer_Geometry.Box;

	function To_Paint (
		Target : SDL_Surface;
		Color  : Video.Colors.Color)
		return   Unsigned_32
	is
	begin
		return Colors.To_Pixel (Target.Data.Pixel_Format, Color);
	end To_Paint;

	procedure Clear (
		Target : in out SDL_Surface;
		Paint  :        Unsigned_32)
	is
	begin
		Target.Data.Fill (
			Area => (
				X => 0,
				Y => 0,
				Width => Target.Data.Size.Width, Height => Target.Data.Size.Height),
			Colour => Paint);
	end Clear;

	procedure Clear (
		Target : in out SDL_Surface;
		Color  :        Video.Colors.Color)
	is
	begin
		Target.Data.Fill (
			Area => (
				X => 0,
				Y => 0,
				Width => Target.Data.Size.Width, Height => Target.Data.Size.Height),
			Colour => Colors.To_Pixel (Target.Data.Pixel_Format, Color));
	end Clear;

	procedure Write_Pixel (
		Target : in out SDL_Surface;
		A      :        Integer_Geometry.Point;
		Paint  :        Unsigned_32)
	is
	begin
		Target.Data.Fill (
			Area => (
				X => SDL.Coordinate (A.X),
				Y => SDL.Coordinate (A.Y),
				Width => 1, Height => 1),
			Colour => Paint);
	end Write_Pixel;

	procedure Write_Pixel (
		Target : in out SDL_Surface;
		A      :        Integer_Geometry.Point;
		Color  :        Video.Colors.Color)
	is
	begin
		Target.Data.Fill (
			Area => (
				X => SDL.Coordinate (A.X),
				Y => SDL.Coordinate (A.Y),
				Width => 1, Height => 1),
			Colour => Colors.To_Pixel (Target.Data.Pixel_Format, Color));
	end Write_Pixel;

	procedure Fill_Rectangle (
		Target : in out SDL_Surface;
		Bounds :        Integer_Geometry.Box;
		Paint  :        Unsigned_32)
	is
	begin
		Target.Data.Fill (
			Area => Rectangles.To_SDL (Bounds),
			Colour => Paint);
	end Fill_Rectangle;

	procedure Fill_Rectangle (
		Target : in out SDL_Surface;
		Bounds :        Integer_Geometry.Box;
		Color  :        Video.Colors.Color)
	is
	begin
		Target.Data.Fill (
			Area => Rectangles.To_SDL (Bounds),
			Colour => Colors.To_Pixel (Target.Data.Pixel_Format, Color));
	end Fill_Rectangle;

	procedure Line (
		Target   : in out SDL_Surface;
		A, B     : in     Integer_Geometry.Point;
		Color    : in     Interfaces.Unsigned_32)
	is
		procedure Put_Pixel (C : Integer_Geometry.Point) is
		begin
			Target.Write_Pixel (C, Color);
		end Put_Pixel;
		procedure Inner_Line is new Algorithms.Generic_Line (Put_Pixel);
	begin
		Inner_Line (A, B, A, B);
	end Line;

	procedure Line (
		Target   : in out SDL_Surface;
		A, B     : in     Integer_Geometry.Point;
		Color    : in     Video.Colors.Color)
	is
	begin
		Target.Line (A, B, Colors.To_Pixel (Target.Data.Pixel_Format, Color));
	end Line;

	procedure Paste (
		Target : in out SDL_Surface;
		A      :        Integer_Geometry.Point;
		Source :        Readable_Surface'Class)
	is
		B : Integer_Geometry.Box := Source.Bounding_Box;
		S, T : SDL.Video.Rectangles.Rectangle;
	begin
		S := SDL2.Rectangles.To_SDL (B);
		T := SDL2.Rectangles.To_SDL (B + A);
		Target.Data.Blit (T, Source.Data, S);
	end Paste;

	procedure Initialize (S : in out Window_Surface) is
	begin
		S.Data := S.Window.Get_Surface;
	end Initialize;

	procedure Finalize (S : in out Window_Surface) is
	begin
		S.Sync;
	end Finalize;

	procedure Sync (S : in out Window_Surface) is
	begin
		S.Window.Update_Surface;
	end Sync;

end Video.SDL2.Surfaces;
