with Ada.Finalization;
with Video.Integer_Geometry;
with Video.Colors;
with Video.Renderables.Color;
with Video.Renderables.Generic_Sprite;
with Video.SDL2.Rectangles;
with SDL.Video.Surfaces;
with SDL.Video.Windows;
use type SDL.Dimension;

package Video.SDL2.Surfaces is

	type Readable_Surface is limited new Ada.Finalization.Limited_Controlled with record
		Data : SDL.Video.Surfaces.Surface;
	end record;

	function Bounding_Box (S : Readable_Surface) return Video.Integer_Geometry.Box
	is (Rectangles.From_SDL (S.Data.Size));

	package Surface_Renderables is new Renderables.Generic_Sprite (Readable_Surface'Class);

	package Pixel_Renderables is new Video.Renderables.Paint_Renderables (Interfaces.Unsigned_32);

	type SDL_Surface is limited new Readable_Surface
	and Pixel_Renderables.Renderable
	and Video.Renderables.Color.Renderable
	and Surface_Renderables.Sprite_Renderable with null record;

	function To_Paint (
		Target : SDL_Surface;
		Color  : Video.Colors.Color)
		return     Interfaces.Unsigned_32;

	overriding procedure Clear (
		Target : in out SDL_Surface;
		Paint  : in     Interfaces.Unsigned_32);

	overriding procedure Clear (
		Target : in out SDL_Surface;
		Color  : in     Video.Colors.Color);

	overriding procedure Write_Pixel (
		Target : in out SDL_Surface;
		A      : in     Integer_Geometry.Point;
		Paint  : in     Interfaces.Unsigned_32);

	overriding procedure Write_Pixel (
		Target : in out SDL_Surface;
		A      : in     Integer_Geometry.Point;
		Color  : in     Video.Colors.Color);

	overriding procedure Fill_Rectangle (
		Target : in out SDL_Surface;
		Bounds : in     Integer_Geometry.Box;
		Paint  : in     Interfaces.Unsigned_32);

	overriding procedure Fill_Rectangle (
		Target : in out SDL_Surface;
		Bounds : in     Integer_Geometry.Box;
		Color  : in     Video.Colors.Color);

	overriding procedure Line (
		Target : in out SDL_Surface;
		A, B   : in     Integer_Geometry.Point;
		Color  : in     Interfaces.Unsigned_32);

	overriding procedure Line (
		Target : in out SDL_Surface;
		A, B   : in     Integer_Geometry.Point;
		Color  : in     Video.Colors.Color);

	overriding procedure Paste (
		Target : in out SDL_Surface;
		A      :        Integer_Geometry.Point;
		Source :        Readable_Surface'Class);

	--

	type Window_Surface (Window : not null access SDL.Video.Windows.Window)
	is limited new SDL_Surface with null record;

	overriding procedure Initialize (S : in out Window_Surface);

	overriding procedure Finalize (S : in out Window_Surface);

	overriding procedure Sync (S : in out Window_Surface);

private

end Video.SDL2.Surfaces;
